import configparser
import os
import random
import time
from datetime import datetime

from common import Material, Feishu, Common, Oss
from common.ffmpeg import FFmpeg
from common.piaoquan_utils import PQ
from common.sql_help import sqlCollect

config = configparser.ConfigParser()
config.read('./config.ini')


class getVideo:

    """
    数据处理
    """
    @classmethod
    def get_video(cls):
        pass

    """
    根据标示+任务标示创建目录
    """
    @classmethod
    def create_folders(cls, mark, task_mark):
        video_path_url = config['PATHS']['VIDEO_PATH'] + mark + "/" + task_mark + "/"
        if not os.path.exists(video_path_url):
            os.makedirs(video_path_url)
        return video_path_url

    """
    随机生成ID
    """
    @classmethod
    def random_id(cls):
        now = datetime.now()
        rand_num = random.randint(10000, 99999)
        oss_id = "{}{}".format(now.strftime("%Y%m%d%H%M%S"), rand_num)
        return oss_id


    """
    删除文件
    """
    @classmethod
    def remove_files(cls, video_path_url):
        if os.path.exists(video_path_url) and os.path.isdir(video_path_url):
            for root, dirs, files in os.walk(video_path_url):
                for file in files:
                    file_path = os.path.join(root, file)
                    os.remove(file_path)
                for dir in dirs:
                    dir_path = os.path.join(root, dir)
                    os.rmdir(dir_path)

    """
    飞书数据处理
    """
    @classmethod
    def video_task(cls, data):
        mark = data["mark"]
        name = data["name"]
        feishu_id = data["feishu_id"]
        feishu_sheet = data["feishu_sheet"]
        pw_sheet = data["pw_sheet"]

        task_data = Material.get_task_data(feishu_id, feishu_sheet)
        if len(task_data) == 0:
            Feishu.bot(mark, '机器自动改造消息通知', f'今日任务为空，请关注', name)
            return mark

        for task in task_data:
            task_marks = task["task_mark"]  # 任务标示
            old_id = str(task["old_id"])
            video_id = str(task["video_id"])
            new_id = str(task["new_id"])
            number = task["number"]  # 指定条数
            title = task["title"]
            video_share = task["video_share"]
            video_ending = task["video_ending"]
            crop_total = task["crop_total"]
            gg_duration_total = task["gg_duration_total"]

            try:
                if ',' in new_id:
                    n_id = new_id.split(',')
                else:
                    n_id = [new_id]
                if old_id and old_id != 'None':
                    task_id = old_id.split(',')
                else:
                    task_id = video_id.split(',')
                values = [task_id[0], task_marks]
                task_mark = "_".join(map(str, values))  # 最终任务标示
                video_path_url = cls.create_folders(mark, str(task_mark))  # 创建目录
                count = 0  # 初始化计数器
                for id in task_id:
                    pw_list, zm_list = Material.get_pwsrt_data(feishu_id, pw_sheet)  # 获取srt
                    zm = random.choice(zm_list)  # 随机选择 视频中字幕
                    pws = random.choice(pw_list)  # 随机选择 片尾srt+音频
                    Common.logger("log").info(f"{task_mark}下的ID{id} 开始获取视频")
                    time.sleep(1)
                    if old_id and old_id != 'None':
                        data_list = PQ.get_user_url(task_mark, id, number, title, mark)
                    else:
                        data_list = PQ.get_audio_url(task_mark, id, title, mark)

                    if len(data_list) == 0:
                        Common.logger("log").info(f"{task_mark}下的视频ID{id} 已经改造过了")
                        Feishu.bot(mark, '机器自动改造消息通知', f'{task_mark}任务下的用户ID{id},没有已经改造的视频了', name)
                        cls.remove_files(video_path_url)
                        continue
                    Common.logger("log").info(f"{task_mark}下的ID{id} 获取视频完成，共{len(data_list)}条")
                    for video in data_list:
                        v_id = video["video_id"]
                        new_title = video["title"]
                        cover = video["cover"]
                        video_url = video["video_url"]
                        time.sleep(1)
                        new_video_path = PQ.download_video(video_url, video_path_url, v_id)  # 下载视频地址
                        if new_video_path == '':
                            Common.logger("log").info(f"{task_mark}下的视频ID{id},{new_video_path}视频下载失败")
                            cls.remove_files(video_path_url)
                            continue
                        # if crop_total and crop_total != 'None':  # 判断是否需要裁剪
                        #     pass
                        # if gg_duration_total and gg_duration_total != 'None':  # 判断是否需要指定视频时长
                        #     pass
                        if video_ending and video_ending != 'None':
                            pw_id = pws["pw_id"]
                            pw_srt = pws["pw_srt"]
                            pw_url = PQ.get_pw_url(pw_id)
                            pw_random_id = cls.random_id()
                            jpg_path = None
                            pw_path = None
                            for attempt in range(3):
                                jpg_path = FFmpeg.video_png(new_video_path, video_path_url, pw_random_id)  # 生成视频最后一帧jpg
                                if os.path.isfile(jpg_path):
                                    Common.logger("log").info(f"{task_mark}下的视频ID{id},生成视频最后一帧成功")
                                    break
                                time.sleep(1)
                            if not os.path.isfile(jpg_path):
                                Feishu.bot(mark, '机器自动改造消息通知', f'{task_mark}任务用户ID{id}下的视频{v_id},获取视频最后一帧失败，请关注', name)
                                cls.remove_files(video_path_url)
                                continue
                            for attempt in range(3):
                                pw_mp3_path = FFmpeg.get_video_mp3(pw_url, video_path_url, pw_random_id)
                                pw_path = FFmpeg.pw_video(jpg_path, video_path_url, pw_url, pw_srt, pw_random_id, pw_mp3_path)  # 生成片尾视频
                                if os.path.isfile(pw_path):
                                    Common.logger("log").info(f"{task_mark}下的视频ID{id},生成片尾视频成功")
                                    break
                                time.sleep(1)
                            if not os.path.isfile(pw_path):
                                Feishu.bot(mark, '机器自动改造消息通知', f'{task_mark}任务用户ID{id}下的视频{v_id},生成片尾视频失败，请关注',
                                           name)
                                cls.remove_files(video_path_url)
                                continue
                            pw_video_list = [new_video_path, pw_path]
                            video_path = FFmpeg.concatenate_videos(pw_video_list, video_path_url)  # 视频与片尾拼接到一起
                            time.sleep(1)
                            if video_share and video_share != 'None':
                                new_video_path = FFmpeg.single_video(video_path, video_share, video_path_url, zm)
                            else:
                                new_video_path = video_path
                        else:
                            if video_share and video_share != 'None':
                                new_video_path = FFmpeg.single_video(new_video_path, video_share, video_path_url, zm)
                        time.sleep(1)
                        oss_id = cls.random_id()
                        oss_object_key = Oss.stitching_sync_upload_oss(new_video_path, oss_id)  # 视频发送OSS
                        status = oss_object_key.get("status")
                        if status == 200:
                            oss_object_key = oss_object_key.get("oss_object_key")
                            time.sleep(1)
                            code = PQ.insert_piaoquantv(oss_object_key, new_title, cover, n_id[count])

                            if code:
                                Common.logger("log").info(f"{task_mark}下的视频ID{v_id}发送成功")
                                sqlCollect.insert_task(task_mark, v_id, mark)  # 插入数据库
                                current_time = datetime.now()
                                formatted_time = current_time.strftime("%Y-%m-%d %H:%M:%S")
                                values = [[name, task_mark, v_id, n_id[count], new_title, str(code), formatted_time]]
                                Feishu.insert_columns("ILb4sa0LahddRktnRipcu2vQnLb", "a74fc4", "ROWS", 1, 2)
                                time.sleep(0.5)
                                Feishu.update_values("ILb4sa0LahddRktnRipcu2vQnLb", "a74fc4", "A2:Z2", values)
                            cls.remove_files(video_path_url)
                        else:
                            cls.remove_files(video_path_url)
                            Common.logger("log").info(f"{task_mark}下的视频ID{id} 视频发送OSS失败 ")
                    count += 1  # 每次迭代计数器加1
                Feishu.bot(mark, '机器自动改造消息通知', f'{task_mark}任务改造完成，请关注', name)
            except Exception as e:
                cls.remove_files(video_path_url)
                Common.logger("warning").warning(f"{name}的{task_mark}任务处理失败:{e}\n")
        return mark
